"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.babelConfig = void 0;

var _defaults = require("../defaults");

var _helpers = require("../helpers");

var _options = require("../options");

// @ts-check

/**
 * @param {Partial<BabelConfigParams>} paramsRaw
 */
const babelConfig = (paramsRaw = {}) => {
  const opts = (0, _options.options)();
  const {
    aliases,
    nodeVersion,
    presetEnvConfig
  } = {
    aliases: opts.aliases,
    nodeVersion: opts.nodeVersion || _defaults.defaults.nodeVersion,

    /**
     * @param {{}} defaultConfig
     */
    presetEnvConfig: defaultConfig => defaultConfig,
    ...paramsRaw
  };
  const src = _defaults.defaults.rootDir;
  const config = {
    presets: [['@babel/preset-env', presetEnvConfig({
      targets: {
        node: nodeVersion
      }
    })], '@babel/preset-typescript'],
    plugins: [!!aliases && ['module-resolver', {
      root: [src],
      alias: Object.keys(aliases).reduce((acc, key) => {
        return { ...acc,
          [key]: (0, _helpers.trimPathSeparator)(aliases[key])
        };
      }, {})
    }], '@babel/proposal-class-properties', '@babel/plugin-proposal-nullish-coalescing-operator', '@babel/plugin-proposal-optional-chaining'].filter(Boolean),
    ignore: [`${src}/**/*.d.ts`, `${src}/**/*.json`]
  };
  return config;
};

exports.babelConfig = babelConfig;

var _default = babelConfig();

exports.default = _default;
//# sourceMappingURL=babel.config.js.map