"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.jestConfig = void 0;

var _defaults = require("../defaults");

var _options = require("../options");

var _helpers = require("../helpers");

// @ts-check

/**
 *
 * @param {Partial<JestConfigParams>} paramsRaw
 */
const jestConfig = (paramsRaw = {}) => {
  const opts = (0, _options.options)();
  const {
    aliases,
    isIntegrationTest
  } = {
    aliases: opts.aliases,
    isIntegrationTest: false,
    ...paramsRaw
  };
  const src = (0, _helpers.ensureStartsWithPathSeparator)(_defaults.defaults.rootDir);
  const lib = (0, _helpers.ensureSurroundedWithPathSeparator)(_defaults.defaults.libOutDir);
  const extensions = _defaults.defaults.extensions;

  const exts = _defaults.defaults.extensions.join(',');

  const moduleNameMapper = aliases && Object.keys(aliases).reduce((acc, key) => {
    const alias = (0, _helpers.trimPathSeparator)(aliases[key]);
    return { ...acc,
      [`^${key}$`]: `<rootDir>/${alias}`,
      [`^${key}/(.*)$`]: `<rootDir>/${alias}/$1`
    };
  }, {});
  const integrationTestMatch = `<rootDir>/${_defaults.defaults.integrationTestsGlob}/*.test.{${exts}}`;
  const unitTestMatch = `<rootDir>/${_defaults.defaults.unitTestsGlob}/*.test.{${exts}}`;
  return {
    testEnvironment: 'node',
    testMatch: isIntegrationTest ? [integrationTestMatch] : [unitTestMatch],
    cacheDirectory: '.jest-cache',
    coverageDirectory: isIntegrationTest ? 'coverage-integration' : 'coverage-unit',
    coverageReporters: ['json', 'lcov', 'text'],
    collectCoverageFrom: [`<rootDir>${src}/**/*.{${exts}}`, `!**/node_modules/**`, `!${_defaults.defaults.integrationTestsGlob}`, `!${_defaults.defaults.unitTestsGlob}`, `!<rootDir>${src}/**/*.d.ts`, `!<rootDir>${src}/**/*.json`],
    moduleFileExtensions: extensions,
    setupFiles: [],
    ...(moduleNameMapper && {
      moduleNameMapper
    }),
    testPathIgnorePatterns: ['/node_modules/', lib],
    modulePathIgnorePatterns: [lib],
    globals: {
      INTEGRATION_TEST: false
    },
    transformIgnorePatterns: ['node_modules', lib, `.*\\.json`, `.*\\.d\\.ts`]
  };
};

exports.jestConfig = jestConfig;
//# sourceMappingURL=jest.config.js.map